@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -39,21 +39,44 @@ G_BEGIN_DECLS
 typedef struct _GMimeFilterBest GMimeFilterBest;
 typedef struct _GMimeFilterBestClass GMimeFilterBestClass;
 
-enum {
-	GMIME_FILTER_BEST_CHARSET      = (1 << 0),
-	GMIME_FILTER_BEST_ENCODING     = (1 << 1)
-};
-
+/**
+ * GMimeFilterBestFlags:
+ * @GMIME_FILTER_BEST_CHARSET: Enable best-charset detection.
+ * @GMIME_FILTER_BEST_ENCODING: Enable best-encoding detection.
+ *
+ * Bit flags to enable charset and/or encoding scanning to make
+ * educated guesses as to what the best charset and/or encodings to
+ * use for the content passed through the filter.
+ **/
 typedef enum {
-	GMIME_BEST_ENCODING_7BIT,
-	GMIME_BEST_ENCODING_8BIT,
-	GMIME_BEST_ENCODING_BINARY
-} GMimeBestEncoding;
+	GMIME_FILTER_BEST_CHARSET  = (1 << 0),
+	GMIME_FILTER_BEST_ENCODING = (1 << 1)
+} GMimeFilterBestFlags;
 
+
+/**
+ * GMimeFilterBest:
+ * @parent_object: parent #GMimeFilter
+ * @flags: #GMimeFilterBestFlags
+ * @charset: #GMimeCharset state
+ * @count0: count of nul-bytes passed through the filter
+ * @count8: count of 8bit bytes passed through the filter
+ * @total: total number of bytes passed through the filter
+ * @maxline: the length of the longest line passed through the filter
+ * @linelen: current line length
+ * @frombuf: buffer for checking From_ lines
+ * @fromlen: length of bytes in @frombuf
+ * @hadfrom: %TRUE if any line started with "From "
+ * @startline: start line state
+ * @midline: mid-line state
+ *
+ * A filter for calculating the best encoding and/or charset to encode
+ * the data passed through it.
+ **/
 struct _GMimeFilterBest {
 	GMimeFilter parent_object;
 	
-	unsigned int flags;
+	GMimeFilterBestFlags flags;
 	
 	/* for best charset detection */
 	GMimeCharset charset;
@@ -82,11 +105,11 @@ struct _GMimeFilterBestClass {
 
 GType g_mime_filter_best_get_type (void);
 
-GMimeFilter *g_mime_filter_best_new (unsigned int flags);
+GMimeFilter *g_mime_filter_best_new (GMimeFilterBestFlags flags);
 
 const char *g_mime_filter_best_charset (GMimeFilterBest *best);
 
-GMimeContentEncoding g_mime_filter_best_encoding (GMimeFilterBest *best, GMimeBestEncoding required);
+GMimeContentEncoding g_mime_filter_best_encoding (GMimeFilterBest *best, GMimeEncodingConstraint constraint);
 
 G_END_DECLS
 
