@@ -1,20 +1,21 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 1999-2006 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2.1
+ *  of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free
+ *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
+ *  02110-1301, USA.
  */
 
 
@@ -35,21 +36,23 @@ G_BEGIN_DECLS
 typedef struct _GMimeFilterCRLF GMimeFilterCRLF;
 typedef struct _GMimeFilterCRLFClass GMimeFilterCRLFClass;
 
-typedef enum {
-	GMIME_FILTER_CRLF_ENCODE,
-	GMIME_FILTER_CRLF_DECODE
-} GMimeFilterCRLFDirection;
-
-typedef enum {
-	GMIME_FILTER_CRLF_MODE_CRLF_DOTS,
-	GMIME_FILTER_CRLF_MODE_CRLF_ONLY
-} GMimeFilterCRLFMode;
-
+/**
+ * GMimeFilterCRLF:
+ * @parent_object: parent #GMimeFilter
+ * @encode: encoding vs decoding line endings/dots
+ * @saw_cr: previous char was a CR
+ * @saw_lf: previous char was a LF
+ * @saw_dot: previous char was a period
+ *
+ * A filter to convert between line-ending formats and encode/decode
+ * lines beginning with a '.'.
+ **/
 struct _GMimeFilterCRLF {
 	GMimeFilter parent_object;
 	
-	GMimeFilterCRLFDirection direction;
-	GMimeFilterCRLFMode mode;
+	gboolean encode;
+	gboolean dots;
+	
 	gboolean saw_cr;
 	gboolean saw_lf;
 	gboolean saw_dot;
@@ -63,7 +66,7 @@ struct _GMimeFilterCRLFClass {
 
 GType g_mime_filter_crlf_get_type (void);
 
-GMimeFilter *g_mime_filter_crlf_new (GMimeFilterCRLFDirection direction, GMimeFilterCRLFMode mode);
+GMimeFilter *g_mime_filter_crlf_new (gboolean encode, gboolean dots);
 
 G_END_DECLS
 
