@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -27,6 +27,7 @@
 
 #include <gmime/gmime-content-type.h>
 #include <gmime/gmime-disposition.h>
+#include <gmime/gmime-encodings.h>
 #include <gmime/gmime-stream.h>
 #include <gmime/gmime-header.h>
 
@@ -76,32 +77,37 @@ struct _GMimeObjectClass {
 	char *       (* get_headers)   (GMimeObject *object);
 	
 	ssize_t      (* write_to_stream) (GMimeObject *object, GMimeStream *stream);
+	
+	void         (* encode) (GMimeObject *object, GMimeEncodingConstraint constraint);
 };
 
 
 /**
- * GMimePartFunc:
- * @part: A MIME part.
+ * GMimeObjectForeachFunc:
+ * @parent: parent #GMimeObject
+ * @part: a #GMimeObject
  * @user_data: User-supplied callback data.
  *
- * The function signature for a callback to
- * g_mime_message_foreach_part() and g_mime_multipart_foreach().
+ * The function signature for a callback to g_mime_message_foreach()
+ * and g_mime_multipart_foreach().
  **/
-typedef void (* GMimePartFunc) (GMimeObject *part, gpointer user_data);
+typedef void (* GMimeObjectForeachFunc) (GMimeObject *parent, GMimeObject *part, gpointer user_data);
 
 
 GType g_mime_object_get_type (void);
 
 void g_mime_object_register_type (const char *type, const char *subtype, GType object_type);
+
+GMimeObject *g_mime_object_new (GMimeContentType *content_type);
 GMimeObject *g_mime_object_new_type (const char *type, const char *subtype);
 
-void g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *mime_type);
-const GMimeContentType *g_mime_object_get_content_type (GMimeObject *object);
+void g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *content_type);
+GMimeContentType *g_mime_object_get_content_type (GMimeObject *object);
 void g_mime_object_set_content_type_parameter (GMimeObject *object, const char *name, const char *value);
 const char *g_mime_object_get_content_type_parameter (GMimeObject *object, const char *name);
 
 void g_mime_object_set_content_disposition (GMimeObject *object, GMimeContentDisposition *disposition);
-const GMimeContentDisposition *g_mime_object_get_content_disposition (GMimeObject *object);
+GMimeContentDisposition *g_mime_object_get_content_disposition (GMimeObject *object);
 
 void g_mime_object_set_disposition (GMimeObject *object, const char *disposition);
 const char *g_mime_object_get_disposition (GMimeObject *object);
@@ -125,6 +131,8 @@ char *g_mime_object_get_headers (GMimeObject *object);
 ssize_t g_mime_object_write_to_stream (GMimeObject *object, GMimeStream *stream);
 char *g_mime_object_to_string (GMimeObject *object);
 
+void g_mime_object_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
+
 G_END_DECLS
 
 #endif /* __GMIME_OBJECT_H__ */
