@@ -22,16 +22,15 @@
 #include <config.h>
 #endif
 
-#include <glib.h>
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_MMAN_H
 #include <sys/mman.h>
+#endif
 #include <unistd.h>
-#include <dirent.h>
 #include <fcntl.h>
 #include <errno.h>
 
@@ -49,7 +48,7 @@ static gboolean
 streams_match (GMimeStream **streams, const char *filename)
 {
 	char buf[4096], dbuf[4096], errstr[1024];
-	size_t totalsize, totalread = 0;
+	gint64 len, totalsize, totalread = 0;
 	size_t nread, size;
 	ssize_t n;
 	
@@ -59,14 +58,14 @@ streams_match (GMimeStream **streams, const char *filename)
 	
 	if (streams[0]->bound_end != -1) {
 		totalsize = streams[0]->bound_end - streams[0]->position;
-	} else if ((n = g_mime_stream_length (streams[0])) == -1) {
-		sprintf (errstr, "Error: Unable to get length of original stream\n");
+	} else if ((len = g_mime_stream_length (streams[0])) == -1) {
+		sprintf (errstr, "Error: Unable to get length of original stream: %s\n", g_strerror (errno));
 		goto fail;
-	} else if (n < (streams[0]->position - streams[0]->bound_start)) {
+	} else if (len < (streams[0]->position - streams[0]->bound_start)) {
 		sprintf (errstr, "Error: Overflow on original stream?\n");
 		goto fail;
 	} else {
-		totalsize = n - (streams[0]->position - streams[0]->bound_start);
+		totalsize = len - (streams[0]->position - streams[0]->bound_start);
 	}
 	
 	while (totalread < totalsize) {
@@ -77,19 +76,19 @@ streams_match (GMimeStream **streams, const char *filename)
 		nread = 0;
 		totalread += n;
 		
-		d(fprintf (stderr, "read " SIZE_T " bytes from stream[0]\n", size));
+		d(fprintf (stderr, "read %zu bytes from stream[0]\n", size));
 		
 		do {
 			if ((n = g_mime_stream_read (streams[1], dbuf + nread, size - nread)) <= 0) {
-				d(fprintf (stderr, "stream[1] read() returned " SSIZE_T ", EOF\n", n));
+				d(fprintf (stderr, "stream[1] read() returned %zd, EOF\n", n));
 				break;
 			}
-			d(fprintf (stderr, "read " SSIZE_T " bytes from stream[1]\n", n));
+			d(fprintf (stderr, "read %zd bytes from stream[1]\n", n));
 			nread += n;
 		} while (nread < size);
 		
 		if (nread < size) {
-			sprintf (errstr, "Error: `%s' appears to be truncated, short %u+ bytes\n",
+			sprintf (errstr, "Error: `%s' appears to be truncated, short %zu+ bytes\n",
 				 filename, size - nread);
 			goto fail;
 		}
@@ -98,7 +97,7 @@ streams_match (GMimeStream **streams, const char *filename)
 			sprintf (errstr, "Error: `%s': content does not match\n", filename);
 			goto fail;
 		} else {
-			d(fprintf (stderr, "%u bytes identical\n", size));
+			d(fprintf (stderr, "%zu bytes identical\n", size));
 		}
 	}
 	
@@ -132,7 +131,7 @@ test_stream_gets (GMimeStream *stream, const char *filename)
 	FILE *fp;
 	
 	if (!(fp = fopen (filename, "r+")))
-		throw (exception_new ("could not open `%s': %s", filename, strerror (errno)));
+		throw (exception_new ("could not open `%s': %s", filename, g_strerror (errno)));
 	
 	while (!g_mime_stream_eos (stream)) {
 		rbuf[0] = '\0';
@@ -146,8 +145,8 @@ test_stream_gets (GMimeStream *stream, const char *filename)
 	fclose (fp);
 	
 	if (strcmp (sbuf, rbuf) != 0) {
-		v(fprintf (stderr, "\tstream: \"%s\" (" SIZE_T ")\n", sbuf, strlen (sbuf)));
-		v(fprintf (stderr, "\treal:   \"%s\" (" SIZE_T ")\n", rbuf, strlen (rbuf)));
+		v(fprintf (stderr, "\tstream: \"%s\" (%zu)\n", sbuf, strlen (sbuf)));
+		v(fprintf (stderr, "\treal:   \"%s\" (%zu)\n", rbuf, strlen (rbuf)));
 		throw (exception_new ("streams did not match"));
 	}
 }
@@ -155,10 +154,10 @@ test_stream_gets (GMimeStream *stream, const char *filename)
 static void
 test_stream_buffer_gets (const char *filename)
 {
-	GMimeStream *stream, *buffered;
+	GMimeStream *stream, *buffered = NULL;
 	int fd;
 	
-	if ((fd = open (filename, O_RDONLY)) == -1) {
+	if ((fd = open (filename, O_RDONLY, 0)) == -1) {
 		v(fprintf (stderr, "failed to open %s", filename));
 		return;
 	}
@@ -203,8 +202,8 @@ test_stream_mem (const char *filename)
 	GMimeStream *stream, *fstream;
 	int fd;
 	
-	if ((fd = open (filename, O_RDONLY)) == -1) {
-		v(fprintf (stderr, "failed to open %s", filename));
+	if ((fd = open (filename, O_RDONLY, 0)) == -1) {
+		v(fprintf (stderr, "failed to open %s: %s\n", filename, g_strerror (errno)));
 		return;
 	}
 	
@@ -246,10 +245,10 @@ check_stream_fs (const char *input, const char *output, const char *filename, gi
 	Exception *ex = NULL;
 	int fd[2];
 	
-	if ((fd[0] = open (input, O_RDONLY)) == -1)
+	if ((fd[0] = open (input, O_RDONLY, 0)) == -1)
 		return FALSE;
 	
-	if ((fd[1] = open (output, O_RDONLY)) == -1) {
+	if ((fd[1] = open (output, O_RDONLY, 0)) == -1) {
 		close (fd[0]);
 		return FALSE;
 	}
@@ -305,6 +304,7 @@ check_stream_file (const char *input, const char *output, const char *filename,
 	return TRUE;
 }
 
+#ifdef HAVE_MMAP
 static gboolean
 check_stream_mmap (const char *input, const char *output, const char *filename, gint64 start, gint64 end)
 {
@@ -312,10 +312,10 @@ check_stream_mmap (const char *input, const char *output, const char *filename,
 	Exception *ex = NULL;
 	int fd[2];
 	
-	if ((fd[0] = open (input, O_RDONLY)) == -1)
+	if ((fd[0] = open (input, O_RDONLY, 0)) == -1)
 		return FALSE;
 	
-	if ((fd[1] = open (output, O_RDONLY)) == -1) {
+	if ((fd[1] = open (output, O_RDONLY, 0)) == -1) {
 		close (fd[0]);
 		return FALSE;
 	}
@@ -337,6 +337,7 @@ check_stream_mmap (const char *input, const char *output, const char *filename,
 	
 	return TRUE;
 }
+#endif /* HAVE_MMAP */
 
 static gboolean
 check_stream_buffer_block (const char *input, const char *output, const char *filename, gint64 start, gint64 end)
@@ -345,10 +346,10 @@ check_stream_buffer_block (const char *input, const char *output, const char *fi
 	Exception *ex = NULL;
 	int fd[2];
 	
-	if ((fd[0] = open (input, O_RDONLY)) == -1)
+	if ((fd[0] = open (input, O_RDONLY, 0)) == -1)
 		return FALSE;
 	
-	if ((fd[1] = open (output, O_RDONLY)) == -1) {
+	if ((fd[1] = open (output, O_RDONLY, 0)) == -1) {
 		close (fd[0]);
 		return FALSE;
 	}
@@ -380,10 +381,10 @@ check_stream_buffer_cache (const char *input, const char *output, const char *fi
 	Exception *ex = NULL;
 	int fd[2];
 	
-	if ((fd[0] = open (input, O_RDONLY)) == -1)
+	if ((fd[0] = open (input, O_RDONLY, 0)) == -1)
 		return FALSE;
 	
-	if ((fd[1] = open (output, O_RDONLY)) == -1) {
+	if ((fd[1] = open (output, O_RDONLY, 0)) == -1) {
 		close (fd[0]);
 		return FALSE;
 	}
@@ -408,6 +409,37 @@ check_stream_buffer_cache (const char *input, const char *output, const char *fi
 	return TRUE;
 }
 
+static gboolean
+check_stream_gio (const char *input, const char *output, const char *filename, gint64 start, gint64 end)
+{
+	GMimeStream *streams[2];
+	Exception *ex = NULL;
+	GFile *file;
+	int fd;
+	
+	if (!(file = g_file_new_for_path (input)))
+		return FALSE;
+	
+	if ((fd = open (output, O_RDONLY, 0)) == -1) {
+		g_object_unref (file);
+		return FALSE;
+	}
+	
+	streams[0] = g_mime_stream_gio_new_with_bounds (file, start, end);
+	streams[1] = g_mime_stream_fs_new (fd);
+	
+	if (!streams_match (streams, filename))
+		ex = exception_new ("GMimeStreamBuffer (Cache Mode) streams did not match for `%s'", filename);
+	
+	g_object_unref (streams[0]);
+	g_object_unref (streams[1]);
+	
+	if (ex != NULL)
+		throw (ex);
+	
+	return TRUE;
+}
+
 
 typedef gboolean (* checkFunc) (const char *, const char *, const char *, gint64, gint64);
 
@@ -417,16 +449,19 @@ static struct {
 } checks[] = {
 	{ "GMimeStreamFs",                  check_stream_fs           },
 	{ "GMimeStreamFile",                check_stream_file         },
+#ifdef HAVE_MMAP
 	{ "GMimeStreamMmap",                check_stream_mmap         },
+#endif /* HAVE_MMAP */
 	{ "GMimeStreamBuffer (block mode)", check_stream_buffer_block },
 	{ "GMimeStreamBuffer (cache mode)", check_stream_buffer_cache },
+	{ "GMimeStreamGIO",                 check_stream_gio          },
 };
 
 static void
-test_streams (DIR *dir, const char *datadir, const char *filename)
+test_streams (GDir *dir, const char *datadir, const char *filename)
 {
 	char inpath[256], outpath[256], *p, *q, *o;
-	struct dirent *dent;
+	const char *dent, *d;
 	gint64 start, end;
 	size_t n;
 	guint i;
@@ -445,38 +480,38 @@ test_streams (DIR *dir, const char *datadir, const char *filename)
 	
 	n = strlen (filename);
 	
-	while ((dent = readdir (dir))) {
-		if (strncmp (dent->d_name, filename, n) != 0 || dent->d_name[n] != ':')
+	while ((dent = g_dir_read_name (dir))) {
+		if (strncmp (dent, filename, n) != 0 || dent[n] != ':')
 			continue;
 		
-		p = dent->d_name + n + 1;
-		if ((start = strtol (p, &o, 10)) < 0 || *o != ',')
+		d = dent + n + 1;
+		if ((start = strtol (d, &o, 10)) < 0 || *o != ',')
 			continue;
 		
-		p = o + 1;
+		d = o + 1;
 		
-		if ((((end = strtol (p, &o, 10)) < start) && end != -1) || *o != '\0')
+		if ((((end = strtol (d, &o, 10)) < start) && end != -1) || *o != '\0')
 			continue;
 		
-		strcpy (q, dent->d_name);
+		strcpy (q, dent);
 		
 		for (i = 0; i < G_N_ELEMENTS (checks); i++) {
-			testsuite_check ("%s on `%s'", checks[i].what, dent->d_name);
+			testsuite_check ("%s on `%s'", checks[i].what, dent);
 			try {
-				if (!checks[i].check (inpath, outpath, dent->d_name, start, end)) {
+				if (!checks[i].check (inpath, outpath, dent, start, end)) {
 					testsuite_check_warn ("%s could not open `%s'",
-							      checks[i].what, dent->d_name);
+							      checks[i].what, dent);
 				} else {
 					testsuite_check_passed ();
 				}
 			} catch (ex) {
 				testsuite_check_failed ("%s on `%s' failed: %s", checks[i].what,
-							dent->d_name, ex->message);
+							dent, ex->message);
 			} finally;
 		}
 	}
 	
-	rewinddir (dir);
+	g_dir_rewind (dir);
 }
 
 
@@ -489,7 +524,7 @@ gen_random_stream (GMimeStream *stream)
 	
 	/* read between 4k and 14k bytes */
 	size = 4096 + (size_t) (10240.0 * (rand () / (RAND_MAX + 1.0)));
-	v(fprintf (stdout, "Generating " SIZE_T " bytes of random data... ", size));
+	v(fprintf (stdout, "Generating %zu bytes of random data... ", size));
 	v(fflush (stdout));
 	
 	while (total < size) {
@@ -520,9 +555,8 @@ gen_test_data (const char *datadir, char **stream_name)
 {
 	GMimeStream *istream, *ostream, *stream;
 	char input[256], output[256], *name, *p;
-	gint64 start, end;
+	gint64 start, end, len;
 	struct stat st;
-	size_t len;
 	int fd, i;
 	
 	srand (time (NULL));
@@ -541,7 +575,7 @@ gen_test_data (const char *datadir, char **stream_name)
 	*name++ = G_DIR_SEPARATOR;
 	strcpy (name, "streamXXXXXX");
 	
-	if ((fd = mkstemp (input)) == -1)
+	if ((fd = g_mkstemp (input)) == -1)
 		return -1;
 	
 	*stream_name = g_strdup (name);
@@ -562,7 +596,7 @@ gen_test_data (const char *datadir, char **stream_name)
 	for (i = 0; i < 64; i++) {
 	retry:
 		start = (gint64) (st.st_size * (rand () / (RAND_MAX + 1.0)));
-		len = (size_t) (st.st_size * (rand () / (RAND_MAX + 1.0)));
+		len = (gint64) (st.st_size * (rand () / (RAND_MAX + 1.0)));
 		if (start + len > st.st_size) {
 			len = st.st_size - start;
 			end = -1;
@@ -591,9 +625,9 @@ int main (int argc, char **argv)
 	const char *datadir = "data/streams";
 	gboolean gen_data = TRUE;
 	char *stream_name = NULL;
-	struct dirent *dent;
 	char path[256], *p;
-	DIR *dir, *outdir;
+	GDir *dir, *outdir;
+	const char *dent;
 	int i;
 	
 	g_mime_init (0);
@@ -613,9 +647,9 @@ int main (int argc, char **argv)
 	*p++ = G_DIR_SEPARATOR;
 	strcpy (p, "output");
 	
-	if (!(outdir = opendir (path))) {
+	if (!(outdir = g_dir_open (path, 0, NULL))) {
 		if (gen_test_data (datadir, &stream_name) == -1 ||
-		    !(outdir = opendir (path)))
+		    !(outdir = g_dir_open (path, 0, NULL)))
 			goto exit;
 		
 		gen_data = FALSE;
@@ -623,10 +657,10 @@ int main (int argc, char **argv)
 	
 	p = g_stpcpy (p, "input");
 	
-	if (!(dir = opendir (path))) {
+	if (!(dir = g_dir_open (path, 0, NULL))) {
 		if (!gen_data || gen_test_data (datadir, &stream_name) == -1 ||
-		    !(dir = opendir (path))) {
-			closedir (outdir);
+		    !(dir = g_dir_open (path, 0, NULL))) {
+			g_dir_close (outdir);
 			goto exit;
 		}
 		
@@ -634,15 +668,15 @@ int main (int argc, char **argv)
 	}
 	
 	if (gen_data) {
-		while ((dent = readdir (dir))) {
-			if (dent->d_name[0] == '.' || !strcmp (dent->d_name, "README"))
+		while ((dent = g_dir_read_name (dir))) {
+			if (dent[0] == '.' || !strcmp (dent, "README"))
 				continue;
 			
 			gen_data = FALSE;
 			break;
 		}
 		
-		rewinddir (dir);
+		g_dir_rewind (dir);
 		
 		if (gen_data && gen_test_data (datadir, &stream_name) == -1)
 			goto exit;
@@ -651,13 +685,13 @@ int main (int argc, char **argv)
 	*p++ = G_DIR_SEPARATOR;
 	*p = '\0';
 	
-	while ((dent = readdir (dir))) {
-		if (dent->d_name[0] == '.' || !strcmp (dent->d_name, "README"))
+	while ((dent = g_dir_read_name (dir))) {
+		if (dent[0] == '.' || !strcmp (dent, "README"))
 			continue;
 		
-		test_streams (outdir, datadir, dent->d_name);
+		test_streams (outdir, datadir, dent);
 		
-		strcpy (p, dent->d_name);
+		strcpy (p, dent);
 		test_stream_buffer_gets (path);
 	}
 	
@@ -671,10 +705,10 @@ int main (int argc, char **argv)
 		p = g_stpcpy (p, "output");
 		*p++ = G_DIR_SEPARATOR;
 		
-		rewinddir (outdir);
-		while ((dent = readdir (outdir))) {
-			if (!strncmp (dent->d_name, stream_name, strlen (stream_name))) {
-				strcpy (p, dent->d_name);
+		g_dir_rewind (outdir);
+		while ((dent = g_dir_read_name (outdir))) {
+			if (!strncmp (dent, stream_name, strlen (stream_name))) {
+				strcpy (p, dent);
 				unlink (path);
 			}
 		}
@@ -682,8 +716,8 @@ int main (int argc, char **argv)
 		g_free (stream_name);
 	}
 	
-	closedir (outdir);
-	closedir (dir);
+	g_dir_close (outdir);
+	g_dir_close (dir);
 	
 exit:
 	
